@@ -1,7 +1,7 @@
 /*
     ccid.h: CCID structures
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2017   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2023   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -97,6 +97,11 @@ typedef struct
 	 */
 	int cardProtocol;
 
+	/*
+	 * Reader protocols
+	 */
+	int dwProtocols;
+
 	/*
 	 * bInterfaceProtocol (CCID, ICCD-A, ICCD-B)
 	 */
@@ -175,6 +180,9 @@ typedef struct
 
 	// True if it is a SAM slot
 	int isSamSlot;
+
+	/* Write delay in milliseconds */
+	int writeDelay;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -246,6 +254,7 @@ typedef struct
 #define VEGAALPHA   0x09820008
 #define HPSMARTCARDKEYBOARD 0x03F01024
 #define HP_CCIDSMARTCARDKEYBOARD 0x03F00036
+#define CHICONYHPSKYLABKEYBOARD 0x04F21469
 #define KOBIL_IDTOKEN 0x0D46301D
 #define FUJITSUSMARTKEYB 0x0BF81017
 #define FEITIANR502DUAL 0x096E060D
@@ -257,9 +266,14 @@ typedef struct
 #define HID_AVIATOR	0x076B3A21
 #define HID_OMNIKEY_5422 0x076B5422
 #define HID_OMNIKEY_3X21 0x076B3031 /* OMNIKEY 3121 or 3021 or 1021 */
+#define HID_OMNIKEY_3821 0x076B3821 /* OMNIKEY 3821 */
 #define HID_OMNIKEY_6121 0x076B6632 /* OMNIKEY 6121 */
 #define CHERRY_XX44	0x046A00A7 /* Cherry Smart Terminal xx44 */
 #define FUJITSU_D323 0x0BF81024 /* Fujitsu Smartcard Reader D323 */
+#define IDENTIV_uTrust3700F		0x04E65790
+#define IDENTIV_uTrust3701F		0x04E65791
+#define IDENTIV_uTrust4701F		0x04E65724
+#define BIT4ID_MINILECTOR		0x25DD3111
 
 // CCID readers
 #define ACS_ACR32_ICC_READER			0x072fb301
@@ -268,6 +282,7 @@ typedef struct
 #define ACS_ACR33U_A3_3SAM_ICC_READER	0x072f8307
 #define ACS_ACR33U_4SAM_ICC_READER		0x072f8301
 #define ACS_ACR39U_SAM_ICC_READER		0x072fb104
+#define ACS_ACR40T_ICC_READER			0x072fb501
 #define ACS_ACR83U						0x072f90d2
 #define ACS_ACR85_PINPAD_READER_ICC		0x072f8306
 #define ACS_ACR85_PINPAD_READER_PICC	0x072f2210
@@ -287,6 +302,7 @@ typedef struct
 #define ACS_ACR1251_1S_DUAL_READER		0x072f2242
 #define ACS_ACR1252_1S_CL_READER		0x072f223b
 #define ACS_ACR1252IMP_1S_CL_READER		0x072f2259
+#define ACS_WALLETMATE_1S_CL_READER		0x072f226b
 #define ACS_ACR1261_1S_DUAL_READER		0x072f2211
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
@@ -301,10 +317,13 @@ typedef struct
 #define ACS_ACR1283_4S_CL_READER		0x072f2213
 #define ACS_ACR1283_CL_READER			0x072f222C
 #define ACS_ACR1283U_FW_UPGRADE			0x072f220C
+#define ACS_ACR1552_1S_CL_READER		0x072f2303
+#define ACS_ACR1581_1S_DUAL_READER		0x072f2301
 #define ACS_AET62_PICC_READER			0x072f0102
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
 #define ACS_APG8201						0x072f8201
 #define ACS_APG8201_B2					0x072f8206
+#define ACS_APG8201_B2RO				0x072f8207
 #define ACS_APG8201Z					0x072f8202
 #define ACS_APG8201Z2					0x072f8205
 
@@ -355,7 +374,7 @@ void acr38_error(int error, const char *file, int line, const char *function);
 void EnablePicc(unsigned int reader_index, int enabled);
 
 /* convert a 4 byte integer in USB format into an int */
-#define dw2i(a, x) (unsigned int)((((((a[x+3] << 8) + a[x+2]) << 8) + a[x+1]) << 8) + a[x])
+#define dw2i(a, x) (unsigned int)(((((((unsigned int)a[x+3] << 8) + (unsigned int)a[x+2]) << 8) + (unsigned int)a[x+1]) << 8) + (unsigned int)a[x])
 
 /* all the data rates specified by ISO 7816-3 Fi/Di tables */
 #define ISO_DATA_RATES 10753, 14337, 15625, 17204, \
