@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2019	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2023	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -90,6 +90,7 @@ typedef struct
 	int bulk_in;
 	int bulk_out;
 	int interrupt;
+	int has_interrupt;
 
 	/* Number of slots using the same device */
 	int real_nb_opened_slots;
@@ -469,7 +470,10 @@ again_libusb:
 					|| (ACS_ACR1251U_C == readerID)
 					|| (ACS_ACR1251K_DUAL_READER == readerID)
 					|| (ACS_ACR1252_1S_CL_READER == readerID)
-					|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
+					|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
+					|| (ACS_WALLETMATE_1S_CL_READER == readerID)
+					|| (ACS_ACR1581_1S_DUAL_READER == readerID)
+					|| (ACS_ACR1552_1S_CL_READER == readerID))
 				{
 						/*
 						 * We can't talk to the two CCID interfaces
@@ -500,6 +504,7 @@ again_libusb:
 					 * 0: R502 Contactless Reader (CCID)
 					 * 1: R502 Contact Reader (CCID)
 					 * 2: R502 SAM1 Reader (CCID)
+					 * 3: R502 SAM2 Reader (CCID)
 					 *
 					 * For the HID Omnikey 5422 the interfaces are:
 					 * 0: OMNIKEY 5422CL Smartcard Reader
@@ -515,9 +520,19 @@ again_libusb:
 						|| (ACS_ACR1251U_C == readerID)
 						|| (ACS_ACR1251K_DUAL_READER == readerID)
 						|| (ACS_ACR1252_1S_CL_READER == readerID)
-						|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
+						|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
+						|| (ACS_WALLETMATE_1S_CL_READER == readerID)
+						|| (ACS_ACR1552_1S_CL_READER == readerID))
 						/* only 2 interfaces for this device */
 						max_interface_number = 1;
+
+					if (ACS_ACR1581_1S_DUAL_READER == readerID)
+						/* 3 interfaces for this device */
+						max_interface_number = 2;
+
+					if (FEITIANR502DUAL == readerID)
+						/* 4 interfaces for Feitian R502 reader */
+						max_interface_number = 3;
 				}
 #endif
 				/* is it already opened? */
@@ -776,7 +791,10 @@ again:
 					|| (ACS_ACR1251U_C == readerID)
 					|| (ACS_ACR1251K_DUAL_READER == readerID)
 					|| (ACS_ACR1252_1S_CL_READER == readerID)
-					|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
+					|| (ACS_ACR1252IMP_1S_CL_READER == readerID)
+					|| (ACS_WALLETMATE_1S_CL_READER == readerID)
+					|| (ACS_ACR1581_1S_DUAL_READER == readerID)
+					|| (ACS_ACR1552_1S_CL_READER == readerID))
 				{
 					/* use the next interface for the next "slot" */
 					static_interface = interface + 1;
@@ -824,6 +842,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = 3;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = 0x07;
+					usbDevice[reader_index].ccid.dwProtocols = 0x0003;
 				}
 				else if ((readerID == ACS_ACR38U_SAM) || (readerID == IRIS_SCR21U) ||
 					(readerID == ACS_AET65_1SAM_ICC_READER))
@@ -843,6 +862,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = 3;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = 0x07;
+					usbDevice[reader_index].ccid.dwProtocols = 0x0003;
 				}
 				else if (readerID == ACS_ACR88U)
 				{
@@ -861,6 +881,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = 3;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = 0x03;
+					usbDevice[reader_index].ccid.dwProtocols = 0x0003;
 				}
 				else if (readerID == ACS_ACR128U)
 				{
@@ -879,6 +900,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = 3;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = 0x03;
+					usbDevice[reader_index].ccid.dwProtocols = 0x0003;
 				}
 				else if ((readerID == ACS_ACR1251_1S_DUAL_READER) ||
 					(readerID == ACS_ACR1261_1S_DUAL_READER) ||
@@ -900,6 +922,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = 3;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = 0x03;
+					usbDevice[reader_index].ccid.dwProtocols = 0x0003;
 				}
 				else
 				{
@@ -937,6 +960,7 @@ again:
 					usbDevice[reader_index].ccid.bNumEndpoints = usb_interface->altsetting->bNumEndpoints;
 					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					usbDevice[reader_index].ccid.bVoltageSupport = device_descriptor[5];
+					usbDevice[reader_index].ccid.dwProtocols = dw2i(device_descriptor, 6);
 				}
 
 				usbDevice[reader_index].ccid.sIFD_serial_number = NULL;
@@ -969,13 +993,18 @@ again:
 				}
 
 				usbDevice[reader_index].ccid.IFD_bcdDevice = desc.bcdDevice;
-#ifndef __APPLE__
-				/* If this is a multislot reader, init the multislot stuff */
-				if (usbDevice[reader_index].ccid.bMaxSlotIndex)
+#ifdef __APPLE__
+				usbDevice[reader_index].multislot_extension = NULL;
+#else
+				/*
+				 * If the reader has interrupt endpoint, init the multislot
+				 * stuff.
+				 */
+				if (usbDevice[reader_index].has_interrupt)
 					usbDevice[reader_index].multislot_extension = Multi_CreateFirstSlot(reader_index);
 				else
-#endif
 					usbDevice[reader_index].multislot_extension = NULL;
+#endif
 
 				libusb_free_config_descriptor(config_desc);
 
@@ -1023,16 +1052,28 @@ again:
 				// Initialize isSamSlot
 				usbDevice[reader_index].ccid.isSamSlot = FALSE;
 
+				/* Initialize the write delay. */
+				usbDevice[reader_index].ccid.writeDelay = 0;
+
 				// The 2nd interface (composite device) is a SAM slot
 				if ((readerID == ACS_ACR1281_1S_PICC_READER) ||
 					(readerID == ACS_ACR1251_1S_CL_READER) ||
 					(readerID == ACS_ACR1251U_C) ||
 					(readerID == ACS_ACR1252_1S_CL_READER) ||
-					(readerID == ACS_ACR1252IMP_1S_CL_READER))
+					(readerID == ACS_ACR1252IMP_1S_CL_READER) ||
+					(readerID == ACS_WALLETMATE_1S_CL_READER) ||
+					(readerID == ACS_ACR1552_1S_CL_READER))
 				{
 					if (interface == 1)
 						usbDevice[reader_index].ccid.isSamSlot = TRUE;
 				}
+
+				// The 3rd interface (composite device) is a SAM slot
+				if (readerID == ACS_ACR1581_1S_DUAL_READER)
+				{
+					if (interface == 2)
+						usbDevice[reader_index].ccid.isSamSlot = TRUE;
+				}
 #ifdef __APPLE__
 				// Initialize terminated flag to false
 				usbDevice[reader_index].terminated = FALSE;
@@ -1067,8 +1108,13 @@ again:
 					goto end2;
 				}
 
-				/* Disable card detection thread for APG8201-B2. */
-				if (usbDevice[reader_index].ccid.readerID == ACS_APG8201_B2)
+				/*
+				 * Disable card detection thread for APG8201-B2 or the reader
+				 * which does not have interrupt endpoint.
+				 */
+				if ((usbDevice[reader_index].ccid.readerID == ACS_APG8201_B2)
+					|| (usbDevice[reader_index].ccid.readerID == ACS_APG8201_B2RO)
+					|| (!usbDevice[reader_index].has_interrupt))
 				{
 					goto end;
 				}
@@ -1153,60 +1199,83 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 	int rv;
 	int actual_length;
 	char debug_header[] = "-> 121234 ";
-	int pos;
-	int len;
-	int delayed = FALSE;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	(void)snprintf(debug_header, sizeof(debug_header), "-> %06X ",
 		(int)reader_index);
 
-	// Fix APG8201 and ACR85 ICC cannot receive command properly
-	// Add delay for APG8201 and ACR85 ICC
-	if ((usbDevice[reader_index].ccid.readerID == ACS_APG8201) ||
-		(usbDevice[reader_index].ccid.readerID == ACS_ACR85_PINPAD_READER_ICC))
-	{
-		delayed = TRUE;
-	}
-
 	// Workaround for ACR122U reader
 	usbDevice[reader_index].last_write_size = length;
 
-	// Send command by dividing number of packets
-	pos = 0;
-	while (length > 0)
+	if ((ccid_descriptor->bInterfaceProtocol == PROTOCOL_ACR38)
+		|| (usbDevice[reader_index].ccid.writeDelay > 0))
 	{
-		if (length > usbDevice[reader_index].bulkOutMaxPacketSize)
-			len = usbDevice[reader_index].bulkOutMaxPacketSize;
-		else
-			len = length;
+		int pos;
+		int len;
 
-		DEBUG_XXD(debug_header, buffer + pos, len);
-
-		rv = libusb_bulk_transfer(usbDevice[reader_index].dev_handle,
-			usbDevice[reader_index].bulk_out, buffer + pos, len,
-			&actual_length, USB_WRITE_TIMEOUT);
-
-		if (rv < 0)
+		// Send command by dividing number of packets
+		pos = 0;
+		while (length > 0)
 		{
-			DEBUG_CRITICAL5("write failed (%d/%d): %d %s",
-				usbDevice[reader_index].bus_number,
-				usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
-
-			if (LIBUSB_ERROR_NO_DEVICE == rv)
-				return STATUS_NO_SUCH_DEVICE;
-
-			return STATUS_UNSUCCESSFUL;
-		}
-
-		if (delayed)
-		{
-			// Delay 10 ms
 			if (length > usbDevice[reader_index].bulkOutMaxPacketSize)
-				usleep(10 * 1000);
+				len = usbDevice[reader_index].bulkOutMaxPacketSize;
+			else
+				len = length;
+
+			if (len <= 0)
+			{
+				DEBUG_CRITICAL2("invalid packet size: %d", len);
+				return STATUS_UNSUCCESSFUL;
+			}
+
+			DEBUG_XXD(debug_header, buffer + pos, len);
+
+			rv = libusb_bulk_transfer(usbDevice[reader_index].dev_handle,
+				usbDevice[reader_index].bulk_out, buffer + pos, len,
+				&actual_length, USB_WRITE_TIMEOUT);
+
+			if (rv < 0)
+			{
+				DEBUG_CRITICAL5("write failed (%d/%d): %d %s",
+					usbDevice[reader_index].bus_number,
+					usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
+
+				if (LIBUSB_ERROR_NO_DEVICE == rv)
+					return STATUS_NO_SUCH_DEVICE;
+
+				return STATUS_UNSUCCESSFUL;
+			}
+
+			/* Delay the write operation. */
+			if ((usbDevice[reader_index].ccid.writeDelay > 0)
+				&& (length > usbDevice[reader_index].bulkOutMaxPacketSize))
+			{
+				usleep(usbDevice[reader_index].ccid.writeDelay * 1000);
+			}
+
+			pos += len;
+			length -= len;
 		}
 
-		pos += len;
-		length -= len;
+		return STATUS_SUCCESS;
+	}
+
+	DEBUG_XXD(debug_header, buffer, length);
+
+	rv = libusb_bulk_transfer(usbDevice[reader_index].dev_handle,
+		usbDevice[reader_index].bulk_out, buffer, length,
+		&actual_length, USB_WRITE_TIMEOUT);
+
+	if (rv < 0)
+	{
+		DEBUG_CRITICAL5("write failed (%d/%d): %d %s",
+			usbDevice[reader_index].bus_number,
+			usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
+
+		if (LIBUSB_ERROR_NO_DEVICE == rv)
+			return STATUS_NO_SUCH_DEVICE;
+
+		return STATUS_UNSUCCESSFUL;
 	}
 
 	return STATUS_SUCCESS;
@@ -1323,7 +1392,7 @@ read_again:
 	DEBUG_XXD(debug_header, buffer, *length);
 
 #define BSEQ_OFFSET 6
-	if ((*length >= BSEQ_OFFSET)
+	if ((*length >= BSEQ_OFFSET +1)
 		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
 	{
 		// Workaround for ACR122U reader
@@ -1523,6 +1592,12 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 
 	usb_interface = get_ccid_usb_interface(desc, &num);
 
+	usbdevice->bulk_in = 0;
+	usbdevice->bulk_out = 0;
+	usbdevice->interrupt = 0;
+	usbdevice->has_interrupt = FALSE;
+	usbdevice->bulkOutMaxPacketSize = 0;
+
 	/*
 	 * 3 Endpoints maximum: Interrupt In, Bulk In, Bulk Out
 	 */
@@ -1534,6 +1609,7 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 		{
 			usbdevice->interrupt =
 				usb_interface->altsetting->endpoint[i].bEndpointAddress;
+			usbdevice->has_interrupt = TRUE;
 			continue;
 		}
 
@@ -1560,12 +1636,33 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 } /* get_end_points */
 
 
+/*****************************************************************************
+ *
+ *					get_ccid_usb_bus_number
+ *
+ ****************************************************************************/
+uint8_t get_ccid_usb_bus_number(int reader_index)
+{
+	return usbDevice[reader_index].bus_number;
+}
+
+
+/*****************************************************************************
+ *
+ *					get_ccid_usb_device_address
+ *
+ ****************************************************************************/
+uint8_t get_ccid_usb_device_address(int reader_index)
+{
+	return usbDevice[reader_index].device_address;
+}
+
 /*****************************************************************************
  *
  *					get_ccid_usb_interface
  *
  ****************************************************************************/
-/*@null@*/ EXTERNAL const struct libusb_interface * get_ccid_usb_interface(
+/*@null@*/ const struct libusb_interface * get_ccid_usb_interface(
 	struct libusb_config_descriptor *desc, int *num)
 {
 	const struct libusb_interface *usb_interface = NULL;
@@ -1647,13 +1744,21 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 	int n, i, len;
 	unsigned char buffer[256*sizeof(int)];	/* maximum is 256 records */
 	unsigned int *uint_array;
+	int bNumDataRatesSupported;
+
+	bNumDataRatesSupported = get_ccid_device_descriptor(get_ccid_usb_interface(desc, &num))[27];
+	if (0 == bNumDataRatesSupported)
+		/* read up to the buffer size */
+		len = sizeof(buffer) / sizeof(int);
+	else
+		len = bNumDataRatesSupported;
 
 	/* See CCID 3.7.3 page 25 */
 	n = ControlUSB(reader_index,
 		0xA1, /* request type */
 		0x03, /* GET_DATA_RATES */
 		0x00, /* value */
-		buffer, sizeof(buffer));
+		buffer, len * sizeof(int));
 
 	/* we got an error? */
 	if (n <= 0)
@@ -1673,8 +1778,7 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 	n /= sizeof(int);
 
 	/* we do not get the expected number of data rates */
-	len = get_ccid_device_descriptor(get_ccid_usb_interface(desc, &num))[27]; /* bNumDataRatesSupported */
-	if ((n != len) && len)
+	if ((n != bNumDataRatesSupported) && bNumDataRatesSupported)
 	{
 		DEBUG_INFO3("Got %d data rates but was expecting %d", n, len);
 
@@ -1894,35 +1998,6 @@ void InterruptStop(int reader_index)
 } /* InterruptStop */
 
 
-/*****************************************************************************
- *
- *					TriggerSlotChange
- *
- ****************************************************************************/
-void TriggerSlotChange(int reader_index)
-{
-	struct usbDevice_MultiSlot_Extension *msExt =
-		usbDevice[reader_index].multislot_extension;
-
-	if (msExt != NULL)
-	{
-		int interrupt_byte =
-			(usbDevice[reader_index].ccid.bCurrentSlotIndex / 4) + 1;
-		int interrupt_value =
-			0x02 << (2 * (usbDevice[reader_index].ccid.bCurrentSlotIndex % 4));
-
-		pthread_mutex_lock(&msExt->mutex);
-
-		msExt->status = LIBUSB_TRANSFER_COMPLETED;
-		memset(msExt->buffer, 0, sizeof(msExt->buffer));
-		msExt->buffer[interrupt_byte] = interrupt_value;
-
-		pthread_cond_broadcast(&msExt->condition);
-		pthread_mutex_unlock(&msExt->mutex);
-	}
-}
-
-
 /*****************************************************************************
  *
  *					Multi_PollingProc
@@ -1951,7 +2026,8 @@ static void *Multi_PollingProc(void *p_ext)
 		if (NULL == transfer)
 		{
 			rv = LIBUSB_ERROR_NO_MEM;
-			DEBUG_COMM2("libusb_alloc_transfer err %d", rv);
+			DEBUG_COMM3("libusb_alloc_transfer err %d %s", rv,
+				libusb_error_name(rv));
 			break;
 		}
 
@@ -1967,7 +2043,8 @@ static void *Multi_PollingProc(void *p_ext)
 		if (rv < 0)
 		{
 			libusb_free_transfer(transfer);
-			DEBUG_COMM2("libusb_submit_transfer err %d", rv);
+			DEBUG_COMM3("libusb_submit_transfer err %d %s", rv,
+				libusb_error_name(rv));
 			break;
 		}
 
@@ -1979,7 +2056,8 @@ static void *Multi_PollingProc(void *p_ext)
 			rv = libusb_handle_events_completed(ctx, &completed);
 			if (rv < 0)
 			{
-				DEBUG_COMM2("libusb_handle_events err %d", rv);
+				DEBUG_COMM3("libusb_handle_events err %d %s", rv,
+					libusb_error_name(rv));
 
 				if (rv == LIBUSB_ERROR_INTERRUPTED)
 					continue;
@@ -2035,6 +2113,52 @@ static void *Multi_PollingProc(void *p_ext)
 						actual_length = 2;
 					}
 
+					/*
+					 * If RDR_to_PC_NotifySlotChange is received, update the
+					 * status for each slot.
+					 */
+					if ((actual_length > 0) && (buffer[0] == 0x50))
+					{
+						int bufferIndex = 0;
+						int bitIndex = 0;
+						int i = 0;
+
+						for (i = 0; i <= usbDevice[msExt->reader_index].ccid.bMaxSlotIndex; i++)
+						{
+							bufferIndex = i / 4;
+							bitIndex = 2 * (i % 4);
+
+							if (bufferIndex + 1 < actual_length)
+							{
+								if (buffer[bufferIndex + 1] & (1 << bitIndex))
+								{
+									usbDevice[msExt->reader_index].ccid.bStatus[i] = CCID_ICC_PRESENT_ACTIVE;
+
+									/* Update the status for SAM slot. */
+									if ((i == 0) && (usbDevice[msExt->reader_index].ccid.isSamSlot))
+									{
+										usbDevice[msExt->reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
+									}
+								}
+								else
+								{
+									usbDevice[msExt->reader_index].ccid.bStatus[i] = CCID_ICC_ABSENT;
+
+									/* Update the status for SAM slot. */
+									if ((i == 0) && (usbDevice[msExt->reader_index].ccid.isSamSlot))
+									{
+										usbDevice[msExt->reader_index].ccid.dwSlotStatus = IFD_ICC_NOT_PRESENT;
+									}
+								}
+
+								DEBUG_INFO5("%d/%d: Slot %d: 0x%02X",
+									usbDevice[msExt->reader_index].bus_number,
+									usbDevice[msExt->reader_index].device_address, i,
+									usbDevice[msExt->reader_index].ccid.bStatus[i]);
+							}
+						}
+					}
+
 					/* log the RDR_to_PC_NotifySlotChange data */
 					slot = 0;
 					for (b=0; b<actual_length-1; b++)
@@ -2052,7 +2176,7 @@ static void *Multi_PollingProc(void *p_ext)
 							change = (slot_status & 2) ? "status changed" : "no change";
 
 							DEBUG_COMM3("slot %d status: %d",
-								s + b*4, slot_status);
+								s + slot, slot_status);
 							DEBUG_COMM3("ICC %s, %s", present, change);
 						}
 						slot += 4;
@@ -2097,9 +2221,10 @@ static void *Multi_PollingProc(void *p_ext)
 
 	if (rv < 0)
 	{
-		DEBUG_CRITICAL4("Multi_PollingProc (%d/%d): error %d",
+		DEBUG_CRITICAL5("Multi_PollingProc (%d/%d): error %d %s",
 			usbDevice[msExt->reader_index].bus_number,
-			usbDevice[msExt->reader_index].device_address, rv);
+			usbDevice[msExt->reader_index].device_address,
+			rv, libusb_error_name(rv));
 	}
 
 	/* Wake up the slot threads so they will exit as well */
@@ -2311,7 +2436,7 @@ static struct usbDevice_MultiSlot_Extension *Multi_CreateNextSlot(int physical_r
 // Card detection thread
 static void *CardDetectionThread(void *pParam)
 {
-	int reader_index = (int) pParam;
+	int reader_index = (int) (intptr_t) pParam;
 	int rv = 0;
 	int status = 0;
 	int actual_length = 0;
@@ -2437,10 +2562,26 @@ static void *CardDetectionThread(void *pParam)
 							if (bufferIndex + 1 < actual_length)
 							{
 								if (buffer[bufferIndex + 1] & (1 << bitIndex))
+								{
 									usbDevice[reader_index].ccid.bStatus[i] = CCID_ICC_PRESENT_ACTIVE;
+
+									/* Update the status for SAM slot. */
+									if ((i == 0) && (usbDevice[reader_index].ccid.isSamSlot))
+									{
+										usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
+									}
+								}
 								else
+								{
 									usbDevice[reader_index].ccid.bStatus[i] = CCID_ICC_ABSENT;
 
+									/* Update the status for SAM slot. */
+									if ((i == 0) && (usbDevice[reader_index].ccid.isSamSlot))
+									{
+										usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_NOT_PRESENT;
+									}
+								}
+
 								DEBUG_INFO5("%d/%d: Slot %d: 0x%02X",
 									usbDevice[reader_index].bus_number,
 									usbDevice[reader_index].device_address, i,
